@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar               |
+   | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar               |
    | Imagemagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -21,6 +21,7 @@
 #include "php_imagick.h"
 #include "php_imagick_defs.h"
 #include "php_imagick_macros.h"
+#include "php_imagick_helpers.h"
 
 #if MagickLibVersion > 0x628
 /* {{{ proto bool ImagickDraw::resetvectorgraphics()
@@ -30,7 +31,7 @@ PHP_METHOD(imagickdraw, resetvectorgraphics)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -49,7 +50,7 @@ PHP_METHOD(imagickdraw, gettextkerning)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -83,7 +84,7 @@ PHP_METHOD(imagickdraw, gettextinterwordspacing)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -111,6 +112,42 @@ PHP_METHOD(imagickdraw, settextinterwordspacing)
 /* }}} */
 #endif
 
+#if MagickLibVersion > 0x655
+/* {{{ proto bool ImagickDraw::getTextInterlineSpacing()
+	Gets the text interword spacing
+*/
+PHP_METHOD(imagickdraw, gettextinterlinespacing)
+{
+	php_imagickdraw_object *internd;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	RETURN_DOUBLE(DrawGetTextInterlineSpacing(internd->drawing_wand));
+}
+/* }}} */
+
+/* {{{ proto bool ImagickDraw::setTextInterlineSpacing(float spacing)
+	Sets the text interword spacing
+*/
+PHP_METHOD(imagickdraw, settextinterlinespacing)
+{
+	php_imagickdraw_object *internd;
+	double spacing;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &spacing) == FAILURE) {
+		return;
+	}
+
+	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	DrawSetTextInterlineSpacing(internd->drawing_wand, spacing);
+	RETURN_TRUE;
+}
+/* }}} */
+#endif
+
 /* {{{ proto ImagickDraw ImagickDraw::__construct()
    The ImagickDraw constructor
 */
@@ -217,7 +254,6 @@ PHP_METHOD(imagickdraw, skewx)
 
 	DrawSkewX(internd->drawing_wand, degrees);
 	RETURN_TRUE;
-
 }
 /* }}} */
 
@@ -238,7 +274,6 @@ PHP_METHOD(imagickdraw, skewy)
 
 	DrawSkewY(internd->drawing_wand, degrees);
 	RETURN_TRUE;
-
 }
 /* }}} */
 
@@ -259,7 +294,6 @@ PHP_METHOD(imagickdraw, translate)
 
 	DrawTranslate(internd->drawing_wand, x, y);
 	RETURN_TRUE;
-
 }
 /* }}} */
 
@@ -270,7 +304,8 @@ PHP_METHOD(imagickdraw, setfillcolor)
 {
 	zval *param;
 	php_imagickdraw_object *internd;
-	php_imagickpixel_object *internp;
+	PixelWand *color_wand;
+	zend_bool allocated;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &param) == FAILURE) {
 		return;
@@ -278,9 +313,55 @@ PHP_METHOD(imagickdraw, setfillcolor)
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	IMAGICK_CAST_PARAMETER_TO_COLOR(param, internp, 2);
-	DrawSetFillColor(internd->drawing_wand, internp->pixel_wand);
+	color_wand = php_imagick_zval_to_pixelwand (param, IMAGICKDRAW_CLASS, &allocated TSRMLS_CC);
+	if (!color_wand)
+		return;
 
+	DrawSetFillColor(internd->drawing_wand, color_wand);
+	if (allocated)
+		color_wand = DestroyPixelWand (color_wand);
+
+	RETURN_TRUE;
+}
+/* }}} */
+
+/* {{{ proto bool ImagickDraw::setResolution(float x, float y)
+	Sets the resolution
+*/
+PHP_METHOD(imagickdraw, setresolution)
+{
+	char *density, *buf = NULL;
+	double x, y;
+	php_imagickdraw_object *internd;
+	DrawInfo *draw_info;
+	DrawingWand *d_wand;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "dd", &x, &y) == FAILURE) {
+		return;
+	}
+
+	internd = (php_imagickdraw_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
+
+	spprintf(&buf, 512, "%fx%f", x, y);
+	density = AcquireString(buf);
+	efree (buf);
+
+	if (!density) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Failed to allocate memory" TSRMLS_CC);
+		return;
+	}
+
+	draw_info          = PeekDrawingWand(internd->drawing_wand);
+	draw_info->density = density;
+
+	d_wand = (DrawingWand *) DrawAllocateWand(draw_info, NULL);
+
+	if (!d_wand) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Failed to allocate new DrawingWand structure" TSRMLS_CC);
+		return;
+	}
+
+	php_imagick_replace_drawingwand(internd, d_wand);
 	RETURN_TRUE;
 }
 /* }}} */
@@ -292,7 +373,8 @@ PHP_METHOD(imagickdraw, setstrokecolor)
 {
 	zval *param;
 	php_imagickdraw_object *internd;
-	php_imagickpixel_object *internp;
+	PixelWand *color_wand;
+	zend_bool allocated;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &param) == FAILURE) {
 		return;
@@ -300,11 +382,15 @@ PHP_METHOD(imagickdraw, setstrokecolor)
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	IMAGICK_CAST_PARAMETER_TO_COLOR(param, internp, 2);
-	DrawSetStrokeColor(internd->drawing_wand, internp->pixel_wand);
+	color_wand = php_imagick_zval_to_pixelwand (param, IMAGICKDRAW_CLASS, &allocated TSRMLS_CC);
+	if (!color_wand)
+		return;
+
+	DrawSetStrokeColor(internd->drawing_wand, color_wand);
+	if (allocated)
+		color_wand = DestroyPixelWand (color_wand);
 
 	RETURN_TRUE;
-
 }
 /* }}} */
 
@@ -324,7 +410,11 @@ PHP_METHOD(imagickdraw, setfillalpha)
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
+#if MagickLibVersion >= 0x635
+	DrawSetFillOpacity(internd->drawing_wand, opacity);
+#else
 	DrawSetFillAlpha(internd->drawing_wand, opacity);
+#endif
 	RETURN_TRUE;
 }
 /* }}} */
@@ -349,7 +439,10 @@ PHP_METHOD(imagickdraw, settextantialias)
 /* }}} */
 
 /* {{{ proto bool ImagickDraw::setTextEncoding(string encoding)
-	Specifies specifies the code set to use for text annotations. The only character encoding which may be specified at this time is "UTF-8" for representing Unicode as a sequence of bytes. Specify an empty string to set text encoding to the system's default. Successful text annotation using Unicode may require fonts designed to support Unicode.
+	Specifies specifies the code set to use for text annotations.
+	The only character encoding which may be specified at this time is "UTF-8" for representing Unicode as a sequence of bytes.
+	Specify an empty string to set text encoding to the system's default. 
+	Successful text annotation using Unicode may require fonts designed to support Unicode.
 */
 PHP_METHOD(imagickdraw, settextencoding)
 {
@@ -384,7 +477,11 @@ PHP_METHOD(imagickdraw, setstrokealpha)
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
+#if MagickLibVersion >= 0x635
+	DrawSetStrokeOpacity(internd->drawing_wand, opacity);
+#else
 	DrawSetStrokeAlpha(internd->drawing_wand, opacity);
+#endif
 	RETURN_TRUE;
 }
 /* }}} */
@@ -415,8 +512,9 @@ PHP_METHOD(imagickdraw, setfont)
 {
 	php_imagickdraw_object *internd;
 	char *font, *absolute;
-	int font_len, error = 0;
+	int font_len;
 	MagickBooleanType status;
+	php_imagick_rw_result_t rc;
 
 	/* Parse parameters given to function */
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &font, &font_len) == FAILURE) {
@@ -425,30 +523,27 @@ PHP_METHOD(imagickdraw, setfont)
 
 	/* Check that no empty string is passed */
 	if (font_len == 0) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Can not set empty font", 2);
+		php_imagick_throw_exception (IMAGICKDRAW_CLASS, "Can not set empty font" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
 	/* And if it wasn't */
-	if (!check_configured_font(font, font_len TSRMLS_CC )) {
+	if (!php_imagick_check_font(font, font_len TSRMLS_CC)) {
 
-		if (!(absolute = expand_filepath(font, NULL TSRMLS_CC))) {
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unable to set font", 2);
-		}
-
-		/* Do a safe-mode check for the font */
-		IMAGICK_SAFE_MODE_CHECK(absolute, error);
-		IMAGICKDRAW_CHECK_READ_OR_WRITE_ERROR(internd, absolute, error, IMAGICK_FREE_FILENAME);
-
-		if (VCWD_ACCESS(absolute, F_OK|R_OK )) {
-			zend_throw_exception_ex(php_imagickdraw_exception_class_entry, 2 TSRMLS_CC,
-				"The given font is not found in the ImageMagick configuration and the file (%s) is not accessible", absolute);
-
-			efree(absolute);
+		if ((absolute = expand_filepath(font, NULL TSRMLS_CC)) == NULL) {
+			php_imagick_throw_exception (IMAGICKDRAW_CLASS, "Unable to set font, file path expansion failed" TSRMLS_CC);
 			return;
 		}
 
+		/* Do an access check for the font */
+		if ((rc = php_imagick_file_access_check (absolute TSRMLS_CC)) != IMAGICK_RW_OK) {
+			// Failed
+			php_imagick_imagickdraw_rw_fail_to_exception (internd->drawing_wand, rc, absolute TSRMLS_CC);
+			efree(absolute);
+			return;
+		}
 		status = DrawSetFont(internd->drawing_wand, absolute);
 		efree(absolute);
 
@@ -456,9 +551,9 @@ PHP_METHOD(imagickdraw, setfont)
 		status = DrawSetFont(internd->drawing_wand, font);
 	}
 
-	/* No magick is going to happen */
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set font", 1);
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set font" TSRMLS_CC);
+		return;
 	}
 
 	RETURN_TRUE;
@@ -482,19 +577,16 @@ PHP_METHOD(imagickdraw, setfontfamily)
 
 	/* Check that no empty string is passed */
 	if (font_family_len == 0) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Can not set empty font family", 2);
-	}
-
-	if (!check_configured_font(font_family, font_family_len TSRMLS_CC )) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unable to set font family; parameter not found in the list of configured fonts", 2);
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Can not set empty font family" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = DrawSetFontFamily(internd->drawing_wand, font_family);
 
-	/* No magick is going to happen */
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set font family", 1);
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set font family" TSRMLS_CC);
+		return;
 	}
 
 	RETURN_TRUE;
@@ -555,15 +647,12 @@ PHP_METHOD(imagickdraw, setfontweight)
 	}
 
 	if (weight >= 100 && weight <= 900) {
-
 		internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-
 		DrawSetFontWeight(internd->drawing_wand, weight);
 		RETURN_TRUE;
-
-	} else {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Font weight valid range is 100-900", 2);
 	}
+	php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Font weight valid range is 100-900" TSRMLS_CC);
+	return;
 }
 /* }}} */
 
@@ -668,7 +757,8 @@ PHP_METHOD(imagickdraw, settextundercolor)
 {
 	zval *param;
 	php_imagickdraw_object *internd;
-	php_imagickpixel_object *internp;
+	PixelWand *color_wand;
+	zend_bool allocated;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &param) == FAILURE) {
 		return;
@@ -676,8 +766,13 @@ PHP_METHOD(imagickdraw, settextundercolor)
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	IMAGICK_CAST_PARAMETER_TO_COLOR(param, internp, 2);
-	DrawSetTextUnderColor(internd->drawing_wand, internp->pixel_wand);
+	color_wand = php_imagick_zval_to_pixelwand (param, IMAGICKDRAW_CLASS, &allocated TSRMLS_CC);
+	if (!color_wand)
+		return;
+
+	DrawSetTextUnderColor(internd->drawing_wand, color_wand);
+	if (allocated)
+		color_wand = DestroyPixelWand (color_wand);
 
 	RETURN_TRUE;
 }
@@ -703,25 +798,26 @@ PHP_METHOD(imagickdraw, setviewbox)
 }
 
 /* {{{ proto string ImagickDraw::getFont()
-	Returns a null-terminaged string specifying the font used when annotating with text. The value returned must be freed by the user when no longer needed.
+	Returns a string specifying the font used when annotating with text
 */
 PHP_METHOD(imagickdraw, getfont)
 {
 	php_imagickdraw_object *internd;
 	char *font;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
 	font = DrawGetFont(internd->drawing_wand);
-	if(font == (char *)NULL || *font == '\0') {
+	
+	if (!font) {
 		RETURN_FALSE;
 	} else {
 		ZVAL_STRING(return_value, font, 1);
-		IMAGICK_FREE_MEMORY(char *, font);
+		IMAGICK_FREE_MAGICK_MEMORY(font);
 		return;
 	}
 }
@@ -735,18 +831,18 @@ PHP_METHOD(imagickdraw, getfontfamily)
 	php_imagickdraw_object *internd;
 	char *font_family;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
 	font_family = DrawGetFontFamily(internd->drawing_wand);
-	if(font_family == (char *)NULL || *font_family == '\0') {
+	if (!font_family) {
 		RETURN_FALSE;
 	} else {
 		ZVAL_STRING(return_value, font_family, 1);
-		IMAGICK_FREE_MEMORY(char *, font_family);
+		IMAGICK_FREE_MAGICK_MEMORY(font_family);
 		return;
 	}
 }
@@ -760,7 +856,7 @@ PHP_METHOD(imagickdraw, getfontsize)
 	php_imagickdraw_object *internd;
 	double font_size;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
@@ -780,7 +876,7 @@ PHP_METHOD(imagickdraw, getfontstyle)
 	php_imagickdraw_object *internd;
 	long font_style;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
@@ -800,7 +896,7 @@ PHP_METHOD(imagickdraw, getfontweight)
 	php_imagickdraw_object *internd;
 	long weight;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
@@ -819,16 +915,11 @@ PHP_METHOD(imagickdraw, clear)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-
-	if (internd->drawing_wand == (DrawingWand *)NULL) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "ImagickDraw object is not allocated properly", 2);
-	}
-
 	ClearDrawingWand(internd->drawing_wand);
 	RETURN_TRUE;
 }
@@ -842,7 +933,7 @@ PHP_METHOD(imagickdraw, gettextdecoration)
 	php_imagickdraw_object *internd;
 	long decoration;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
@@ -862,52 +953,23 @@ PHP_METHOD(imagickdraw, gettextencoding)
 	php_imagickdraw_object *internd;
 	char *encoding;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	encoding = DrawGetTextEncoding(internd->drawing_wand);
 
-	if(encoding == (char *)NULL || *encoding == '\0') {
+	if (!encoding) {
 		RETURN_FALSE;
 	} else {
 		ZVAL_STRING(return_value, encoding, 1);
-		IMAGICK_FREE_MEMORY(char *, encoding);
+		IMAGICK_FREE_MAGICK_MEMORY(encoding);
 		return;
 	}
 }
 /* }}} */
 
-/* {{{ proto bool ImagickDraw::destroy()
-	Frees all resources associated with the drawing wand.
-*/
-PHP_METHOD(imagickdraw, destroy)
-{
-	zval *object;
-	php_imagickdraw_object *internd;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
-		return;
-	}
-	
-	object = getThis();
-	internd = (php_imagickdraw_object *)zend_object_store_get_object(object TSRMLS_CC);
-
-	if (internd->drawing_wand == (DrawingWand *)NULL) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "ImagickDraw object is not allocated properly", 2);
-	}
-
-	ClearDrawingWand(internd->drawing_wand);
-#ifdef Z_SET_REFCOUNT_P
-	Z_SET_REFCOUNT_P(object, 0);
-#else
-	object->refcount = 0;
-#endif
-	RETURN_TRUE;
-}
-/* }}} */
-
 /* {{{ proto bool ImagickDraw::annotation(float x, float y, string *text)
 	Draws text on the image.
 */
@@ -931,8 +993,9 @@ PHP_METHOD(imagickdraw, annotation)
 	font = DrawGetFont(internd->drawing_wand);
 
 	/* Fixes PECL Bug #11328 */
-	if(font == (char *)NULL || *font == '\0') {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Font needs to be set before annotating an image", 2);
+	if (!font) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Font needs to be set before annotating an image" TSRMLS_CC);
+		return;
 	}
 #endif
 
@@ -997,11 +1060,11 @@ PHP_METHOD(imagickdraw, polygon)
 		return;
 	}
 
-	coordinates = get_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
+	coordinates = php_imagick_zval_to_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
 
-	if (coordinates == (PointInfo *)NULL) {
-		efree(coordinates);
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unable to read coordinate array", 2);
+	if (!coordinates) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Unable to read coordinate array" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
@@ -1028,11 +1091,11 @@ PHP_METHOD(imagickdraw, bezier)
 		return;
 	}
 
-	coordinates = get_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
+	coordinates = php_imagick_zval_to_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
 
-	if (coordinates == (PointInfo *)NULL) {
-		efree(coordinates);
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unable to read coordinate array", 2);
+	if (!coordinates) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Unable to read coordinate array" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
@@ -1091,16 +1154,23 @@ PHP_METHOD(imagickdraw, clone)
 	php_imagickdraw_object *internd, *intern_return;
 	DrawingWand *tmp_wand;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
+	IMAGICK_METHOD_DEPRECATED("ImagickDraw", "clone");
+
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	tmp_wand = CloneDrawingWand(internd->drawing_wand);
 
+	if (!tmp_wand) {
+		php_imagick_throw_exception (IMAGICK_CLASS, "Failed to allocate DrawingWand structure" TSRMLS_CC);
+		return;
+	}
+
 	object_init_ex(return_value, php_imagickdraw_sc_entry);
 	intern_return = (php_imagickdraw_object *)zend_object_store_get_object(return_value TSRMLS_CC);
-	IMAGICKDRAW_REPLACE_DRAWINGWAND(intern_return, tmp_wand);
+	php_imagick_replace_drawingwand(intern_return, tmp_wand);
 
 	return;
 }
@@ -1126,7 +1196,7 @@ PHP_METHOD(imagickdraw, affine)
 	}
 
 	/* Allocate space to build matrix */
-	pmatrix = (AffineMatrix *)emalloc(sizeof(AffineMatrix ));
+	pmatrix = emalloc(sizeof(AffineMatrix));
 
 	affine = Z_ARRVAL_P(affine_matrix);
 	zend_hash_internal_pointer_reset_ex(affine, (HashPosition *) 0);
@@ -1134,20 +1204,18 @@ PHP_METHOD(imagickdraw, affine)
 	for (i = 0; i < 6 ; i++) {
 
 		if (zend_hash_find(affine, matrix_elements[i], 3, (void**)&ppzval) == FAILURE) {
-
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "AffineMatrix should contain keys: sx, rx, ry, sy, tx and ty", 2);
-
-		} else if ((Z_TYPE_PP(ppzval ) != IS_DOUBLE ) && (Z_TYPE_PP(ppzval ) != IS_LONG )) {
-
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "AffineMatrix values should be ints or floats", 2);
-		
+			efree(pmatrix);
+			php_imagick_throw_exception(IMAGICKDRAW_CLASS, "AffineMatrix must contain keys: sx, rx, ry, sy, tx and ty" TSRMLS_CC);
+			return;
 		} else {
+			zval tmp_zval, *tmp_pzval;
 
-			if(Z_TYPE_PP(ppzval) == IS_LONG) {
-				value = (double) Z_LVAL_PP(ppzval);
-			} else {
-				value = Z_DVAL_PP(ppzval);
-			}
+			tmp_zval = **ppzval;
+			zval_copy_ctor(&tmp_zval);
+			tmp_pzval = &tmp_zval;
+			convert_to_double(tmp_pzval);
+
+			value = Z_DVAL(tmp_zval);
 
 			if (strcmp(matrix_elements[i], "sx") == 0) {
 				pmatrix->sx = value;
@@ -1161,8 +1229,6 @@ PHP_METHOD(imagickdraw, affine)
 				pmatrix->tx = value;
 			} else if (strcmp(matrix_elements[i], "ty") == 0) {
 				pmatrix->ty = value;
-			} else {
-				IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unkown key in AffineMatrix", 2);
 			}
 		}
 	}
@@ -1194,13 +1260,16 @@ PHP_METHOD(imagickdraw, composite)
 	}
 
 	intern = (php_imagick_object *) zend_object_store_get_object(magick_obj TSRMLS_CC);
-	IMAGICK_CHECK_NOT_EMPTY(intern->magick_wand, 1, 1);
+
+	if (php_imagick_ensure_not_empty (intern->magick_wand) == 0)
+		return;
 
 	internd = (php_imagickdraw_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = DrawComposite(internd->drawing_wand, compose, x, y, width, height, intern->magick_wand);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to composite", 2);
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Compositing image failed" TSRMLS_CC);
+		return;
 	}
 
 	RETURN_TRUE;
@@ -1257,18 +1326,18 @@ PHP_METHOD(imagickdraw, getclippath)
 	php_imagickdraw_object *internd;
 	char *clip_path;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	clip_path = DrawGetClipPath(internd->drawing_wand);
 
-	if(clip_path == (char *)NULL || *clip_path == '\0') {
+	if (!clip_path) {
 		RETURN_FALSE;
 	} else {
 		ZVAL_STRING(return_value, clip_path, 1);
-		IMAGICK_FREE_MEMORY(char *, clip_path);
+		IMAGICK_FREE_MAGICK_MEMORY(clip_path);
 		return;
 	}
 }
@@ -1282,7 +1351,7 @@ PHP_METHOD(imagickdraw, getcliprule)
 	php_imagickdraw_object *internd;
 	long clip_rule;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1301,7 +1370,7 @@ PHP_METHOD(imagickdraw, getclipunits)
 	php_imagickdraw_object *internd;
 	long units;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1321,7 +1390,7 @@ PHP_METHOD(imagickdraw, getfillcolor)
 	php_imagickdraw_object *internd;
 	PixelWand *tmp_wand;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1332,7 +1401,7 @@ PHP_METHOD(imagickdraw, getfillcolor)
 
 	object_init_ex(return_value, php_imagickpixel_sc_entry);
 	internp = (php_imagickpixel_object *) zend_object_store_get_object(return_value TSRMLS_CC);
-	IMAGICKPIXEL_REPLACE_PIXELWAND(internp, tmp_wand);
+	php_imagick_replace_pixelwand(internp, tmp_wand);
 
 	return;
 }
@@ -1346,7 +1415,7 @@ PHP_METHOD(imagickdraw, getfillopacity)
 	php_imagickdraw_object *internd;
 	double opacity;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1365,7 +1434,7 @@ PHP_METHOD(imagickdraw, getfillrule)
 	php_imagickdraw_object *internd;
 	long fill_rule;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1384,7 +1453,7 @@ PHP_METHOD(imagickdraw, getgravity)
 	php_imagickdraw_object *internd;
 	long gravity;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
@@ -1403,7 +1472,7 @@ PHP_METHOD(imagickdraw, getstrokeantialias)
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1427,7 +1496,7 @@ PHP_METHOD(imagickdraw, getstrokecolor)
 	php_imagickdraw_object *internd;
 	PixelWand *tmp_wand;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1438,7 +1507,7 @@ PHP_METHOD(imagickdraw, getstrokecolor)
 
 	object_init_ex(return_value, php_imagickpixel_sc_entry);
 	internp = (php_imagickpixel_object *) zend_object_store_get_object(return_value TSRMLS_CC);
-	IMAGICKPIXEL_REPLACE_PIXELWAND(internp, tmp_wand);
+	php_imagick_replace_pixelwand(internp, tmp_wand);
 
 	return;
 }
@@ -1451,9 +1520,10 @@ PHP_METHOD(imagickdraw, getstrokedasharray)
 {
 	php_imagickdraw_object *internd;
 	double *stroke_array;
-	unsigned long num_elements, i;
+	unsigned long i;
+	size_t num_elements;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1466,7 +1536,7 @@ PHP_METHOD(imagickdraw, getstrokedasharray)
 		add_next_index_double(return_value, stroke_array[i]);
 	}
 
-	IMAGICK_FREE_MEMORY(double *, stroke_array);
+	IMAGICK_FREE_MAGICK_MEMORY(stroke_array);
 	return;
 }
 /* }}} */
@@ -1486,10 +1556,11 @@ PHP_METHOD(imagickdraw, setstrokedasharray)
 		return;
 	}
 
-	double_array = get_double_array_from_zval(param_array, &elements TSRMLS_CC);
+	double_array = php_imagick_zval_to_double_array(param_array, &elements TSRMLS_CC);
 
-	if (double_array == (double *)NULL) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Can't read array", 2);
+	if (!double_array) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Cannot read stroke dash array parameter" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
@@ -1509,7 +1580,7 @@ PHP_METHOD(imagickdraw, getstrokedashoffset)
 	php_imagickdraw_object *internd;
 	double offset;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1528,7 +1599,7 @@ PHP_METHOD(imagickdraw, getstrokelinecap)
 	php_imagickdraw_object *internd;
 	long line_cap;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1547,7 +1618,7 @@ PHP_METHOD(imagickdraw, getstrokelinejoin)
 	php_imagickdraw_object *internd;
 	long line_join;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1566,7 +1637,7 @@ PHP_METHOD(imagickdraw, getstrokemiterlimit)
 	php_imagickdraw_object *internd;
 	unsigned long miter_limit;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1585,7 +1656,7 @@ PHP_METHOD(imagickdraw, getstrokeopacity)
 	php_imagickdraw_object *internd;
 	double opacity;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1604,7 +1675,7 @@ PHP_METHOD(imagickdraw, getstrokewidth)
 	php_imagickdraw_object *internd;
 	double width;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1623,7 +1694,7 @@ PHP_METHOD(imagickdraw, gettextalignment)
 	php_imagickdraw_object *internd;
 	long align_type;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1642,7 +1713,7 @@ PHP_METHOD(imagickdraw, gettextantialias)
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1665,7 +1736,7 @@ PHP_METHOD(imagickdraw, getvectorgraphics)
 	php_imagickdraw_object *internd;
 	char *vector;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1673,7 +1744,7 @@ PHP_METHOD(imagickdraw, getvectorgraphics)
 	vector = DrawGetVectorGraphics(internd->drawing_wand);
 
 	ZVAL_STRING(return_value, vector, 1);
-	IMAGICK_FREE_MEMORY(char *, vector);
+	IMAGICK_FREE_MAGICK_MEMORY(vector);
 
 	return;
 }
@@ -1688,18 +1759,23 @@ PHP_METHOD(imagickdraw, gettextundercolor)
 	php_imagickdraw_object *internd;
 	PixelWand *tmp_wand;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
-	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
+	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	tmp_wand = NewPixelWand();
+
+	if (!tmp_wand) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Failed to allocate space for new PixelWand" TSRMLS_CC);
+		return;
+	}
+
 	DrawGetTextUnderColor(internd->drawing_wand, tmp_wand);
 
 	object_init_ex(return_value, php_imagickpixel_sc_entry);
 	internp = (php_imagickpixel_object *) zend_object_store_get_object(return_value TSRMLS_CC);
-	IMAGICKPIXEL_REPLACE_PIXELWAND(internp, tmp_wand);
+	php_imagick_replace_pixelwand(internp, tmp_wand);
 
 	return;
 }
@@ -1712,7 +1788,7 @@ PHP_METHOD(imagickdraw, pathclose)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -1931,7 +2007,7 @@ PHP_METHOD(imagickdraw, pathfinish)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -2108,7 +2184,7 @@ PHP_METHOD(imagickdraw, pathstart)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -2133,11 +2209,11 @@ PHP_METHOD(imagickdraw, polyline)
 		return;
 	}
 
-	coordinates = get_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
+	coordinates = php_imagick_zval_to_pointinfo_array(coordinate_array, &num_elements TSRMLS_CC);
 
-	if (coordinates == (PointInfo *)NULL) {
-		efree(coordinates);
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKDRAW_CLASS, "Unable to read coordinate array", 2);
+	if (!coordinates) {
+		php_imagick_throw_exception(IMAGICKDRAW_CLASS, "Unable to read coordinate array" TSRMLS_CC);
+		return;
 	}
 
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
@@ -2156,7 +2232,7 @@ PHP_METHOD(imagickdraw, popclippath)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -2173,7 +2249,7 @@ PHP_METHOD(imagickdraw, popdefs)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -2191,18 +2267,17 @@ PHP_METHOD(imagickdraw, poppattern)
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = DrawPopPattern(internd->drawing_wand);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to terminate the pattern definition", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to terminate the pattern definition" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2234,7 +2309,7 @@ PHP_METHOD(imagickdraw, pushdefs)
 {
 	php_imagickdraw_object *internd;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -2266,31 +2341,35 @@ PHP_METHOD(imagickdraw, pushpattern)
 }
 /* }}} */
 
-/* {{{ proto bool Imagick::render()
+/* {{{ proto bool ImagickDraw::render()
 	Renders all preceding drawing commands.
 */
 PHP_METHOD(imagickdraw, render)
 {
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
-	char *old_locale = NULL, *buffer = NULL;
-	zend_bool restore = 0;
+	char *old_locale;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	IMAGICK_SET_LOCALE(old_locale, buffer, restore);
+	old_locale = php_imagick_set_locale (TSRMLS_C);
+
 	status = DrawRender(internd->drawing_wand);
-	IMAGICK_RESTORE_LOCALE(old_locale, restore);
+
+	php_imagick_restore_locale (old_locale);
+
+	if (old_locale)
+		efree (old_locale);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to render the drawing commands", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to render the drawing commands" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2353,10 +2432,10 @@ PHP_METHOD(imagickdraw, setclippath)
 	status = DrawSetClipPath(internd->drawing_wand, clip_mask);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set clipping path", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set clipping path" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2421,7 +2500,8 @@ PHP_METHOD(imagickdraw, setfillopacity)
 /* }}} */
 
 /* {{{ proto bool ImagickDraw::setFillPatternURL(string fill_url)
-	Sets the URL to use as a fill pattern for filling objects. Only local URLs ("#identifier") are supported at this time. These local URLs are normally created by defining a named fill pattern with DrawPushPattern/DrawPopPattern.
+	Sets the URL to use as a fill pattern for filling objects. Only local URLs ("#identifier") are supported at this time.
+	These local URLs are normally created by defining a named fill pattern with DrawPushPattern/DrawPopPattern.
 */
 PHP_METHOD(imagickdraw, setfillpatternurl)
 {
@@ -2434,15 +2514,15 @@ PHP_METHOD(imagickdraw, setfillpatternurl)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &url, &url_len) == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = DrawSetFillPatternURL(internd->drawing_wand, url);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set fill pattern URL", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set fill pattern URL" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2500,15 +2580,15 @@ PHP_METHOD(imagickdraw, setstrokepatternurl)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &url, &url_len) == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *) zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = DrawSetStrokePatternURL(internd->drawing_wand, url);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set the stroke pattern URL", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set stroke pattern URL" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2631,10 +2711,10 @@ PHP_METHOD(imagickdraw, setvectorgraphics)
 	status = DrawSetVectorGraphics(internd->drawing_wand, vector);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to set the vector graphics", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to set the vector graphics" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2646,18 +2726,18 @@ PHP_METHOD(imagickdraw, pop)
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = PopDrawingWand(internd->drawing_wand);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to pop the current ImagickDraw object", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to pop the current ImagickDraw object" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
@@ -2669,18 +2749,18 @@ PHP_METHOD(imagickdraw, push)
 	php_imagickdraw_object *internd;
 	MagickBooleanType status;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internd = (php_imagickdraw_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = PushDrawingWand(internd->drawing_wand);
 
 	if (status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKDRAW_EXCEPTION(internd->drawing_wand, "Unable to push the current ImagickDraw object", 2);
-	} else {
-		RETURN_TRUE;
+		php_imagick_convert_imagickdraw_exception (internd->drawing_wand, "Unable to push the current ImagickDraw object" TSRMLS_CC);
+		return;
 	}
+	RETURN_TRUE;
 }
 /* }}} */
 
