@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar               |
+   | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar               |
    | Imagemagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -21,6 +21,7 @@
 #include "php_imagick.h"
 #include "php_imagick_defs.h"
 #include "php_imagick_macros.h"
+#include "php_imagick_helpers.h"
 
 #if MagickLibVersion > 0x628
 /* {{{ proto array ImagickPixel::getHSL()
@@ -31,7 +32,7 @@ PHP_METHOD(imagickpixel, gethsl)
 	php_imagickpixel_object *internp;
 	double hue, saturation, luminosity;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -67,13 +68,15 @@ PHP_METHOD(imagickpixel, sethsl)
 }
 /* }}} */
 
-/* {{{ proto int ImagickPixel::getColorValueQuantum(int color)
-	Gets the quantum color of the ImagickPixel
+/* {{{ proto Quantum ImagickPixel::getColorValueQuantum(int color)
+	Gets the quantum value of a color in the ImagickPixel. Quantum is a float if ImageMagick was compiled with HDRI
+	otherwise an integer.
 */
 PHP_METHOD(imagickpixel, getcolorvaluequantum)
 {
 	php_imagickpixel_object *internp;
-	long color, color_value;
+	long color;
+	Quantum color_value;
 
 	/* Parse parameters given to function */
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &color) == FAILURE) {
@@ -84,47 +87,53 @@ PHP_METHOD(imagickpixel, getcolorvaluequantum)
 	
 	switch (color) {
 
-		case IMAGICKCOLORBLACK:
+		case PHP_IMAGICK_COLOR_BLACK:
 			color_value = PixelGetBlackQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORBLUE:
+		case PHP_IMAGICK_COLOR_BLUE:
 			color_value = PixelGetBlueQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORCYAN:
+		case PHP_IMAGICK_COLOR_CYAN:
 			color_value = PixelGetCyanQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORGREEN:
+		case PHP_IMAGICK_COLOR_GREEN:
 			color_value = PixelGetGreenQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORRED:
+		case PHP_IMAGICK_COLOR_RED:
 			color_value = PixelGetRedQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORYELLOW:
+		case PHP_IMAGICK_COLOR_YELLOW:
 			color_value = PixelGetYellowQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORMAGENTA:
+		case PHP_IMAGICK_COLOR_MAGENTA:
 			color_value = PixelGetMagentaQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLOROPACITY:
+		case PHP_IMAGICK_COLOR_OPACITY:
 			color_value = PixelGetOpacityQuantum(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORALPHA:
+		case PHP_IMAGICK_COLOR_ALPHA:
 			color_value = PixelGetAlphaQuantum(internp->pixel_wand);
 		break;
 
 		default:
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "Unknown color type", 4);
+			php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+			return;
 		break;
 	}
-	RETVAL_LONG(color_value);		
+	
+#ifdef MAGICKCORE_HDRI_ENABLE
+	RETVAL_DOUBLE(color_value);
+#else
+	RETVAL_LONG(color_value);
+#endif
 }
 /* }}} */
 
@@ -134,55 +143,66 @@ PHP_METHOD(imagickpixel, getcolorvaluequantum)
 PHP_METHOD(imagickpixel, setcolorvaluequantum)
 {
 	php_imagickpixel_object *internp;
-	long color, color_value;
+	long color;
 
+
+#ifdef MAGICKCORE_HDRI_ENABLE
+	double color_value;
+	/* Parse parameters given to function */
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ld", &color, &color_value) == FAILURE) {
+		return;
+	}
+#else
+	long color_value;
 	/* Parse parameters given to function */
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &color, &color_value) == FAILURE) {
 		return;
 	}
+#endif
 	
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
 	switch (color) {
 
-		case IMAGICKCOLORBLACK:
+		case PHP_IMAGICK_COLOR_BLACK:
 			PixelSetBlackQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORBLUE:
+		case PHP_IMAGICK_COLOR_BLUE:
 			PixelSetBlueQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORCYAN:
+		case PHP_IMAGICK_COLOR_CYAN:
 			PixelSetCyanQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORGREEN:
+		case PHP_IMAGICK_COLOR_GREEN:
 			PixelSetGreenQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORRED:
+		case PHP_IMAGICK_COLOR_RED:
 			PixelSetRedQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORYELLOW:
+		case PHP_IMAGICK_COLOR_YELLOW:
 			PixelSetYellowQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORMAGENTA:
+		case PHP_IMAGICK_COLOR_MAGENTA:
 			PixelSetMagentaQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLOROPACITY:
+		case PHP_IMAGICK_COLOR_OPACITY:
 			PixelSetOpacityQuantum(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORALPHA:
+		case PHP_IMAGICK_COLOR_ALPHA:
 			PixelSetAlphaQuantum(internp->pixel_wand, color_value);
 		break;
 
 		default:
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "Unknown color type", 4);
+			php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+			return;
 		break;
 	}
 	RETVAL_TRUE;
@@ -196,7 +216,7 @@ PHP_METHOD(imagickpixel, getindex)
 {
 	php_imagickpixel_object *internp;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
@@ -231,27 +251,28 @@ PHP_METHOD(imagickpixel, setindex)
 PHP_METHOD(imagickpixel, __construct)
 {
 	php_imagickpixel_object *internp;
-	MagickBooleanType status;
-	char *color_name = (char *)0;
+	char *color_name = NULL;
 	int color_name_len = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &color_name, &color_name_len) == FAILURE )
-	{
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &color_name, &color_name_len) == FAILURE) {
 		return;
 	}
 
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	internp->pixel_wand = NewPixelWand();
 
+	if (!internp->pixel_wand) {
+		php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Failed to allocate PixelWand structure" TSRMLS_CC);
+		return;
+	}
+
 	/* If color was given as parameter, set it here.*/
-	if (color_name_len != 0 && color_name != (char *)0 )
-	{
-		status = PixelSetColor(internp->pixel_wand, color_name);
-		if(status == MagickFalse) {
-			IMAGICK_THROW_IMAGICKPIXEL_EXCEPTION(internp->pixel_wand, "Unable to construct ImagickPixel", 4);
+	if (color_name && color_name_len) {
+		if (PixelSetColor(internp->pixel_wand, color_name) == MagickFalse) {
+			php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unable to construct ImagickPixel" TSRMLS_CC);
+			return;
 		}
 	}
-	RETURN_TRUE;
 }
 /* }}} */
 
@@ -267,16 +288,16 @@ PHP_METHOD(imagickpixel, setcolor)
 	MagickBooleanType status;
 
 	/* Parse parameters given to function */
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &color_name, &color_name_len) == FAILURE )
-	{
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &color_name, &color_name_len) == FAILURE) {
 		return;
 	}
 	
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	status = PixelSetColor(internp->pixel_wand, color_name);
 
-	if(status == MagickFalse) {
-		IMAGICK_THROW_IMAGICKPIXEL_EXCEPTION(internp->pixel_wand, "Unable to set ImagickPixel color", 4);
+	if (status == MagickFalse) {
+		php_imagick_convert_imagickpixel_exception (internp->pixel_wand, "Unable to set ImagickPixel color" TSRMLS_CC);
+		return;
 	}
 
 	RETURN_TRUE;
@@ -290,59 +311,29 @@ PHP_METHOD(imagickpixel, clear)
 {
 	php_imagickpixel_object *internp;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (!internp->pixel_wand) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "ImagickPixel is not allocated", 4);
-	}
-
 	ClearPixelWand(internp->pixel_wand);
 	RETURN_TRUE;
 }
 /* }}} */
 
-/* {{{ proto bool ImagickPixel::destroy()
-	Deallocates resources associated with a PixelWand.
-*/
-PHP_METHOD(imagickpixel, destroy)
-{
-	zval *object;
-	php_imagickpixel_object *internp;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
-		return;
-	}
-	
-	object = getThis();
-	internp = (php_imagickpixel_object *)zend_object_store_get_object(object TSRMLS_CC);
-
-	if (!internp->pixel_wand) {
-		IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "ImagickPixel is not allocated properly", 4);
-	}
-
-	ClearPixelWand(internp->pixel_wand);
-#ifdef Z_SET_REFCOUNT_P
-	Z_SET_REFCOUNT_P(object, 0);
-#else
-	object->refcount = 0;
-#endif
-	RETURN_TRUE;
-}
-/* }}} */
-
 /* {{{ proto bool ImagickPixel::isSimilar(float fuzz)
 	Returns true if the distance between two colors is less than the specified distance.
 */
-PHP_METHOD(imagickpixel, issimilar)
+static
+void s_is_pixelwand_similar(INTERNAL_FUNCTION_PARAMETERS, zend_bool use_quantum)
 {
 	zval *param;
 	double fuzz;
-	php_imagickpixel_object *internp, *internp_second;
+	php_imagickpixel_object *internp;
 	MagickBooleanType status;
+	PixelWand *color_wand;
+	zend_bool allocated;
 
 	/* Parse parameters given to function */
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zd", &param, &fuzz) == FAILURE) {
@@ -350,23 +341,53 @@ PHP_METHOD(imagickpixel, issimilar)
 	}
 
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
-	IMAGICK_CAST_PARAMETER_TO_COLOR(param, internp_second, 4);
 
-	status = IsPixelWandSimilar(internp->pixel_wand, internp_second->pixel_wand, fuzz);
+	color_wand = php_imagick_zval_to_pixelwand (param, IMAGICKPIXEL_CLASS, &allocated TSRMLS_CC);
+	if (!color_wand)
+		return;
 
-	if(status == MagickFalse) {
+	status = IsPixelWandSimilar(internp->pixel_wand, color_wand, (use_quantum ? (QuantumRange * fuzz) : fuzz));
+	if (allocated)
+		color_wand = DestroyPixelWand (color_wand);
+
+	if (status == MagickFalse) {
 		RETURN_FALSE;
 	}
 
 	RETURN_TRUE;
 }
+
+
+/* {{{ proto bool ImagickPixel::isPixelSimilarQuantum(ImagickPixel pixel, float fuzz)
+	Returns true if the distance between two colors is less than the specified distance.
+	The fuzz value should be in the range 0-QuantumRange.
+	The maximum value represents the longest possible distance in the colorspace.
+	e.g. from RGB(0, 0, 0) to RGB(255, 255, 255) for the RGB colorspace
+*/
+PHP_METHOD(imagickpixel, ispixelsimilarquantum)
+{
+	s_is_pixelwand_similar (INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
+}
+/* }}} */
+
+/* {{{ proto bool ImagickPixel::isPixelSimilar(ImagickPixel pixel, float fuzz)
+	Returns true if the distance between two colors is less than the specified distance.
+	The fuzz value should be in the range 0-1.
+	The maximum value represents the longest possible distance in the colorspace.
+	e.g. from RGB(0, 0, 0) to RGB(255, 255, 255) for the RGB colorspace
+*/
+PHP_METHOD(imagickpixel, ispixelsimilar)
+{
+	s_is_pixelwand_similar (INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
+}
 /* }}} */
 
 /* {{{ proto float ImagickPixel::getColorValue(int color)
-	Gets the normalized color of the ImagickPixel.
+	Gets the normalized value of a color in the ImagickPixel.
 */
 PHP_METHOD(imagickpixel, getcolorvalue)
 {
+	php_imagick_color_t color_enum;
 	php_imagickpixel_object *internp;
 	long color;
 	double color_value = 0;
@@ -378,52 +399,60 @@ PHP_METHOD(imagickpixel, getcolorvalue)
 
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	switch (color) {
+	if (color <= PHP_IMAGICK_COLOR_MIN || color >= PHP_IMAGICK_COLOR_MAX) {
+		php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+		return;
+	}
 
-		case IMAGICKCOLORBLACK:
+	color_enum = color;
+
+	switch (color_enum) {
+
+		case PHP_IMAGICK_COLOR_BLACK:
 			color_value = PixelGetBlack(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORBLUE:
+		case PHP_IMAGICK_COLOR_BLUE:
 			color_value = PixelGetBlue(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORCYAN:
+		case PHP_IMAGICK_COLOR_CYAN:
 			color_value = PixelGetCyan(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORGREEN:
+		case PHP_IMAGICK_COLOR_GREEN:
 			color_value = PixelGetGreen(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORRED:
+		case PHP_IMAGICK_COLOR_RED:
 			color_value = PixelGetRed(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORYELLOW:
+		case PHP_IMAGICK_COLOR_YELLOW:
 			color_value = PixelGetYellow(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORMAGENTA:
+		case PHP_IMAGICK_COLOR_MAGENTA:
 			color_value = PixelGetMagenta(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLOROPACITY:
+		case PHP_IMAGICK_COLOR_OPACITY:
 			color_value = PixelGetOpacity(internp->pixel_wand);
 		break;
 
-		case IMAGICKCOLORALPHA:
+		case PHP_IMAGICK_COLOR_ALPHA:
 			color_value = PixelGetAlpha(internp->pixel_wand);
 		break;
 
 #if MagickLibVersion > 0x628
-		case IMAGICKCOLORFUZZ:
+		case PHP_IMAGICK_COLOR_FUZZ:
 			color_value = PixelGetFuzz(internp->pixel_wand);
 		break;
 #endif
 
 		default:
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "Unknown color type", 4);
+			php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+			return;
 		break;
 	}
 	RETVAL_DOUBLE(color_value);
@@ -435,6 +464,7 @@ PHP_METHOD(imagickpixel, getcolorvalue)
 */
 PHP_METHOD(imagickpixel, setcolorvalue)
 {
+	php_imagick_color_t color_enum;
 	php_imagickpixel_object *internp;
 	long color;
 	double color_value;
@@ -443,107 +473,180 @@ PHP_METHOD(imagickpixel, setcolorvalue)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ld", &color, &color_value) == FAILURE) {
 		return;
 	}
-	
+
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	switch (color) {
+	if (color <= PHP_IMAGICK_COLOR_MIN || color >= PHP_IMAGICK_COLOR_MAX) {
+		php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+		return;
+	}
 
-		case IMAGICKCOLORBLACK:
+	color_enum = color;
+
+	switch (color_enum) {
+
+		case PHP_IMAGICK_COLOR_BLACK:
 			PixelSetBlack(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORBLUE:
+		case PHP_IMAGICK_COLOR_BLUE:
 			PixelSetBlue(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORCYAN:
+		case PHP_IMAGICK_COLOR_CYAN:
 			PixelSetCyan(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORGREEN:
+		case PHP_IMAGICK_COLOR_GREEN:
 			PixelSetGreen(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORRED:
+		case PHP_IMAGICK_COLOR_RED:
 			PixelSetRed(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORYELLOW:
+		case PHP_IMAGICK_COLOR_YELLOW:
 			PixelSetYellow(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORMAGENTA:
+		case PHP_IMAGICK_COLOR_MAGENTA:
 			PixelSetMagenta(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLOROPACITY:
+		case PHP_IMAGICK_COLOR_OPACITY:
 			PixelSetOpacity(internp->pixel_wand, color_value);
 		break;
 
-		case IMAGICKCOLORALPHA:
+		case PHP_IMAGICK_COLOR_ALPHA:
 			PixelSetAlpha(internp->pixel_wand, color_value);
 		break;
 
 #if MagickLibVersion > 0x628
-		case IMAGICKCOLORFUZZ:
+		case PHP_IMAGICK_COLOR_FUZZ:
 			PixelSetFuzz(internp->pixel_wand, color_value);
 		break;
 #endif
 
 		default:
-			IMAGICK_THROW_EXCEPTION_WITH_MESSAGE(IMAGICKPIXEL_CLASS, "Unknown color type", 4);
+			php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Unknown color type" TSRMLS_CC);
+			return;
 		break;
 	}
 	RETVAL_TRUE;
 }
 /* }}} */
 
-/* {{{ proto array ImagickPixel::getColor([bool normalized] )
+/* {{{ proto array ImagickPixel::getColor([int normalization])
 	Returns the color of the pixel in an array
+	normalization - 0 - values returned in the range 0,255 and will be ints, except 
+		for legacy reasons alpha which is 0-1
+	normalization - 1 - values returned in the range 0,1 and will be floats
+	normalization - 2 - values returned in the range 0,255 and will be ints including alpha
+	values i.e. float if ImageMagick was compiled with HDRI, or integers normally.
 */
 PHP_METHOD(imagickpixel, getcolor)
 {
 	php_imagickpixel_object *internp;
-	zend_bool normalized = 0;
+	long normalization = 0;
 	double red, green, blue, alpha;
 
 	/* Parse parameters given to function */
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &normalized) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &normalization) == FAILURE) {
 		return;
 	}
 
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 	array_init(return_value);
 
-	if (normalized == 1) {
+	red   = PixelGetRed(internp->pixel_wand);
+	green = PixelGetGreen(internp->pixel_wand);
+	blue  = PixelGetBlue(internp->pixel_wand);
+	alpha = PixelGetAlpha(internp->pixel_wand);
 
-		red = PixelGetRed(internp->pixel_wand);
-		green = PixelGetGreen(internp->pixel_wand);
-		blue = PixelGetBlue(internp->pixel_wand);
-		alpha = PixelGetAlpha(internp->pixel_wand);
+	switch (normalization){
+		//values returned in the range 0,255 and will be ints
+		case(0): { 
+			//Leave like this for legacy code 
+			//TODO fix the alpha not being normalised at next major/minor verysion
+			red   *= 255;
+			green *= 255;
+			blue  *= 255;
 
-		add_assoc_double(return_value, "r", red);
-		add_assoc_double(return_value, "g", green);
-		add_assoc_double(return_value, "b", blue);
-		add_assoc_double(return_value, "a", alpha);
-	
-	} else {
+			//values are always >=0, so the rounding below may not be necessary
+			add_assoc_long(return_value, "r", (long) (red   > 0.0 ? red   + 0.5 : red   - 0.5));
+			add_assoc_long(return_value, "g", (long) (green > 0.0 ? green + 0.5 : green - 0.5));
+			add_assoc_long(return_value, "b", (long) (blue  > 0.0 ? blue  + 0.5 : blue  - 0.5));
+			add_assoc_long(return_value, "a", alpha);
+			break;
+		}
 
-		red = PixelGetRed(internp->pixel_wand ) * 255;
-		green = PixelGetGreen(internp->pixel_wand ) * 255;
-		blue = PixelGetBlue(internp->pixel_wand ) * 255;
-		alpha = PixelGetAlpha(internp->pixel_wand);
+		//values returned in the range 0,1 and will be floats
+		case(1): {
+			add_assoc_double(return_value, "r", red);
+			add_assoc_double(return_value, "g", green);
+			add_assoc_double(return_value, "b", blue);
+			add_assoc_double(return_value, "a", alpha);
+			break;
+		}
 
-		add_assoc_long(return_value, "r", (int)(red > 0.0 ? red + 0.5 : red - 0.5));
-		add_assoc_long(return_value, "g", (int)(green > 0.0 ? green + 0.5 : green - 0.5));
-		add_assoc_long(return_value, "b", (int)(blue > 0.0 ? blue + 0.5 : blue - 0.5));
-		add_assoc_long(return_value, "a", alpha);
+		case(2): {
+			red   *= 255;
+			green *= 255;
+			blue  *= 255;
+			alpha *= 255;
+
+			//values are always >=0, so the rounding below may not be necessary
+			add_assoc_long(return_value, "r", (long) (red   > 0.0 ? red   + 0.5 : red   - 0.5));
+			add_assoc_long(return_value, "g", (long) (green > 0.0 ? green + 0.5 : green - 0.5));
+			add_assoc_long(return_value, "b", (long) (blue  > 0.0 ? blue  + 0.5 : blue  - 0.5));
+			add_assoc_long(return_value, "a", (long) (alpha  > 0.0 ? alpha  + 0.5 : alpha  - 0.5));
+			break;
+		}
 	}
 
 	return;
 }
 /* }}} */
 
+
+/* {{{ proto array ImagickPixel::getColorQuantum()
+	Returns the color of the pixel in an array as Quantum values. If ImageMagick was compiled
+	as HDRI these will be floats, otherwise they will be integers
+*/
+PHP_METHOD(imagickpixel, getcolorquantum)
+{
+	php_imagickpixel_object *internp;
+	Quantum red, green, blue, alpha;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
+	array_init(return_value);
+
+	red   = PixelGetRedQuantum(internp->pixel_wand);
+	green = PixelGetGreenQuantum(internp->pixel_wand);
+	blue  = PixelGetBlueQuantum(internp->pixel_wand)  ;
+	alpha = PixelGetAlphaQuantum(internp->pixel_wand);
+
+
+#ifdef MAGICKCORE_HDRI_ENABLE
+	add_assoc_double(return_value, "r", red);
+	add_assoc_double(return_value, "g", green);
+	add_assoc_double(return_value, "b", blue);
+	add_assoc_double(return_value, "a", alpha);
+#else
+	add_assoc_long(return_value, "r", (long)red);
+	add_assoc_long(return_value, "g", (long)green);
+	add_assoc_long(return_value, "b", (long)blue);
+	add_assoc_long(return_value, "a", (long)alpha);
+#endif
+
+	return;
+}
+/* }}} */
+
 /* {{{ proto array ImagickPixel::getColorAsString(void )
         Returns the color as a string
 */
@@ -552,21 +655,21 @@ PHP_METHOD(imagickpixel, getcolorasstring)
 	php_imagickpixel_object *internp;
 	char *color_string;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
 	color_string = PixelGetColorAsString(internp->pixel_wand);
 	ZVAL_STRING(return_value, color_string, 1);
 
-	IMAGICK_FREE_MEMORY(char *, color_string);
+	IMAGICK_FREE_MAGICK_MEMORY(color_string);
 	return;
 }
 /* }}} */
 
-/* {{{ proto array ImagickPixel::clone(void )
+/* {{{ proto ImagickPixel ImagickPixel::clone(void)
         Clones the ImagickPixel
 */
 PHP_METHOD(imagickpixel, clone)
@@ -575,18 +678,24 @@ PHP_METHOD(imagickpixel, clone)
 	php_imagickpixel_object *intern_return;
 	PixelWand *pixel_wand;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
-	
+
+	IMAGICK_METHOD_DEPRECATED("ImagickPixel", "clone");
+
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	IMAGICK_CLONE_PIXELWAND(internp->pixel_wand, pixel_wand);
+	pixel_wand = php_imagick_clone_pixelwand (internp->pixel_wand);
+	if (!pixel_wand) {
+		php_imagick_throw_exception (IMAGICKPIXEL_CLASS, "Failed to allocate" TSRMLS_CC);
+		return;
+	}
 
 	object_init_ex(return_value, php_imagickpixel_sc_entry);
 	intern_return = (php_imagickpixel_object *)zend_object_store_get_object(return_value TSRMLS_CC);
 
-	IMAGICKPIXEL_REPLACE_PIXELWAND(intern_return, pixel_wand);
+	php_imagick_replace_pixelwand(intern_return, pixel_wand);
 	return;
 }
 /* }}} */
@@ -597,20 +706,18 @@ PHP_METHOD(imagickpixel, clone)
 PHP_METHOD(imagickpixel, getcolorcount)
 {
 	php_imagickpixel_object *internp;
-	long color_count;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 	
 	internp = (php_imagickpixel_object *)zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	color_count = PixelGetColorCount(internp->pixel_wand);
-	RETVAL_LONG(color_count);
+	RETVAL_LONG(PixelGetColorCount(internp->pixel_wand));
 }
 /* }}} */
 
-/* {{{ proto int ImagickPixel::setColorCount(int colorCount )
+/* {{{ proto int ImagickPixel::setColorCount(int colorCount)
 	Sets the color count associated with this color.
 */
 PHP_METHOD(imagickpixel, setcolorcount)
