@@ -23,8 +23,8 @@
 
 /* Define Extension Properties */
 #define PHP_IMAGICK_EXTNAME    "imagick"
-#define PHP_IMAGICK_VERSION    "3.1.1"
-#define PHP_IMAGICK_EXTNUM     30101
+#define PHP_IMAGICK_VERSION    "3.3.0"
+#define PHP_IMAGICK_EXTNUM     30300
 
 /* Import configure options when building 
    outside of the PHP source tree */
@@ -39,32 +39,8 @@
 /* Include PHP Standard Headers */
 #include "php.h"
 
-/* Include locale header */
-#ifdef HAVE_LOCALE_H
-# include <locale.h>
-#endif
-
-/* Include magic wand header */
-#ifdef IMAGICK_USE_NEW_HEADER
-#  include <wand/MagickWand.h>
-#else
-#  include <wand/magick-wand.h>
-#endif
-
-#ifndef DefaultChannels
-# define DefaultChannels (AllChannels &~ OpacityChannel)
-#endif
-
-/* Some extra headers */
-#include "Zend/zend_exceptions.h"
-#include "Zend/zend_interfaces.h"
-#include "ext/standard/php_string.h"
-#include "ext/standard/info.h"
-#include "ext/standard/php_filestat.h"
-#include "php_ini.h"
-
 /* Define the entry point symbol
- * Zend will use when loading this module
+ * ZE will use when loading this module
  */
 extern zend_module_entry imagick_module_entry;
 #define phpext_imagick_ptr &imagick_module_entry
