@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5 / Imagick	                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2006-2009 Mikko Koppanen, Scott MacVicar               |
+   | Copyright (c) 2006-2013 Mikko Koppanen, Scott MacVicar               |
    | Imagemagick (c) ImageMagick Studio LLC                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
@@ -18,13 +18,49 @@
    +----------------------------------------------------------------------+
 */
 #ifndef PHP_IMAGICK_DEFS_H /* PHP_IMAGICK_DEFS_H */
-/* Prevent double inclusion */
-#define PHP_IMAGICK_DEFS_H
+# define PHP_IMAGICK_DEFS_H
+
+/* Include magic wand header */
+#if defined (IM_MAGICKWAND_HEADER_STYLE_SEVEN)
+#  include <MagickWand/MagickWand.h>
+#elif defined (IM_MAGICKWAND_HEADER_STYLE_OLD)
+#  include <wand/magick-wand.h>
+#else
+#  include <wand/MagickWand.h>
+#endif
+
+/* Some extra headers */
+#include "Zend/zend_exceptions.h"
+#include "Zend/zend_interfaces.h"
+#include "ext/standard/php_string.h"
+#include "ext/standard/info.h"
+#include "ext/standard/php_filestat.h"
+#include "php_ini.h"
+
+/* Include locale header */
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#if MagickLibVersion >= 0x680
+	#define IMAGICK_WITH_KERNEL
+#endif
+
+typedef struct _php_imagick_callback {
+	void ***thread_ctx;
+	zval *user_callback;
+	struct _php_imagick_callback *previous_callback;
+} php_imagick_callback;
 
 /* Globals, needed for the ini settings */
 ZEND_BEGIN_MODULE_GLOBALS(imagick)
 	zend_bool locale_fix;
 	zend_bool progress_monitor;
+	zend_bool skip_version_check;
+	php_imagick_callback *progress_callback;
+#ifdef PHP_IMAGICK_ZEND_MM
+	MagickWand *keep_alive;
+#endif
 ZEND_END_MODULE_GLOBALS(imagick)
 
 #ifdef ZTS
@@ -43,6 +79,10 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 # endif
 #endif
 
+#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 3
+#define zend_parse_parameters_none() zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "")
+#endif
+
 /* Class names */
 #define PHP_IMAGICK_SC_NAME "Imagick"
 #define PHP_IMAGICK_EXCEPTION_SC_NAME "ImagickException"
@@ -56,14 +96,21 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 #define PHP_IMAGICKPIXELITERATOR_SC_NAME "ImagickPixelIterator"
 #define PHP_IMAGICKPIXELITERATOR_EXCEPTION_SC_NAME "ImagickPixelIteratorException"
 
+#ifdef IMAGICK_WITH_KERNEL
+#define PHP_IMAGICKKERNEL_SC_NAME "ImagickKernel"
+#define PHP_IMAGICKKERNEL_EXCEPTION_SC_NAME "ImagickKernelException"
+#endif
+
+
 /* Structure for Imagick object. */
 typedef struct _php_imagick_object  {
 	zend_object zo;
 	MagickWand *magick_wand;
 	char *progress_monitor_name;
-	int next_out_of_bound;
+	zend_bool next_out_of_bound;
 } php_imagick_object;
 
+
 /* Structure for ImagickDraw object. */
 typedef struct _php_imagickdraw_object  {
 	zend_object zo;
@@ -74,8 +121,8 @@ typedef struct _php_imagickdraw_object  {
 typedef struct _php_imagickpixeliterator_object  {
 	zend_object zo;
 	PixelIterator *pixel_iterator;
-	long instanciated_correctly;
-	int iterator_type;
+	zend_bool initialized;
+
 #if MagickLibVersion <= 0x628
 	long rows;
 	long iterator_position;
@@ -84,76 +131,57 @@ typedef struct _php_imagickpixeliterator_object  {
 
 /* Structure for ImagickPixel object. */
 typedef struct _php_imagickpixel_object  {
-    zend_object zo;
-    PixelWand *pixel_wand;
-	int initialized_via_iterator;
+	zend_object zo;
+	PixelWand *pixel_wand;
+	zend_bool initialized_via_iterator;
 } php_imagickpixel_object;
 
-
-void initialize_imagick_constants();
-void add_assoc_string_helper(zval *retvalue, char *name, char *key, char *hash_value TSRMLS_DC );
-void deallocate_wands(MagickWand *magick, DrawingWand *draw, PixelWand *pixel TSRMLS_DC );
-void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC);
-#if MagickLibVersion <= 0x628
-void count_pixeliterator_rows(php_imagickpixeliterator_object *internpix TSRMLS_DC );
+#ifdef IMAGICK_WITH_KERNEL
+/* Structure for ImagickKernel object. */
+typedef struct _php_imagickkernel_object  {
+    zend_object zo;
+    KernelInfo *kernel_info;
+} php_imagickkernel_object;
 #endif
 
-int count_occurences_of(char needle, char *haystack TSRMLS_DC );
-int read_image_into_magickwand(php_imagick_object *intern, int type, char *filename, int filename_len TSRMLS_DC);
-int write_image_from_filename(php_imagick_object *intern, char *filename, zend_bool adjoin, int type TSRMLS_DC);
-int check_configured_font(char *font, int font_len TSRMLS_DC);
-int check_write_access(char *absolute TSRMLS_DC);
-
-MagickBooleanType php_imagick_read_image_using_imagemagick(php_imagick_object *intern, int type, char *filename, int filename_len TSRMLS_DC);
-
-
-
-zend_bool crop_thumbnail_image(MagickWand *magick_wand, long desired_width, long desired_height TSRMLS_DC);
-char *get_pseudo_filename(char* pseudo_string TSRMLS_DC);
-double *get_double_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
-long *get_long_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
-unsigned char *get_char_array_from_zval(zval *param_array, long *num_elements TSRMLS_DC);
-
-MagickBooleanType php_imagick_progress_monitor(const char *text, const MagickOffsetType offset, const MagickSizeType span, void *client_data);
-int php_imagick_stream_handler(php_imagick_object *intern, php_stream *stream, char *filename, int type TSRMLS_DC);
-zend_bool php_imagick_validate_map(const char *map TSRMLS_DC);
-
 /* Define some color constants */
-#define IMAGICKCOLORBLACK 11
-#define IMAGICKCOLORBLUE 12
-#define IMAGICKCOLORCYAN 13
-#define IMAGICKCOLORGREEN 14
-#define IMAGICKCOLORRED 15
-#define IMAGICKCOLORYELLOW 16
-#define IMAGICKCOLORMAGENTA 17
-#define IMAGICKCOLOROPACITY 18
-#define IMAGICKCOLORALPHA 19
-#define IMAGICKCOLORFUZZ 20
+typedef enum _php_imagick_color_t {
+	PHP_IMAGICK_COLOR_MIN = 10,
+	PHP_IMAGICK_COLOR_BLACK,
+	PHP_IMAGICK_COLOR_BLUE,
+	PHP_IMAGICK_COLOR_CYAN,
+	PHP_IMAGICK_COLOR_GREEN,
+	PHP_IMAGICK_COLOR_RED,
+	PHP_IMAGICK_COLOR_YELLOW,
+	PHP_IMAGICK_COLOR_MAGENTA,
+	PHP_IMAGICK_COLOR_OPACITY,
+	PHP_IMAGICK_COLOR_ALPHA,
+	PHP_IMAGICK_COLOR_FUZZ,
+	PHP_IMAGICK_COLOR_MAX,
+} php_imagick_color_t;
 
 /* Class enum */
-#define IMAGICK_CLASS 1
-#define IMAGICKDRAW_CLASS 2
-#define IMAGICKPIXELITERATOR_CLASS 3
-#define IMAGICKPIXEL_CLASS 4
-
-/* Free filename constants */
-#define IMAGICK_DONT_FREE_FILENAME 0
-#define IMAGICK_FREE_FILENAME 1
+typedef enum _php_imagick_class_type_t {
+	IMAGICK_CLASS,
+	IMAGICKDRAW_CLASS,
+	IMAGICKPIXELITERATOR_CLASS,
+	IMAGICKPIXEL_CLASS,
+#ifdef IMAGICK_WITH_KERNEL
+	IMAGICKKERNEL_CLASS
+#endif
+} php_imagick_class_type_t;
 
 /* Read / write constants */
-#define IMAGICK_READ_WRITE_NO_ERROR 0
-#define IMAGICK_READ_WRITE_SAFE_MODE_ERROR 1
-#define IMAGICK_READ_WRITE_OPEN_BASEDIR_ERROR 2
-#define IMAGICK_READ_WRITE_UNDERLYING_LIBRARY 3
-#define IMAGICK_READ_WRITE_PERMISSION_DENIED 4
-#define IMAGICK_READ_WRITE_FILENAME_TOO_LONG 5
-#define IMAGICK_READ_WRITE_PATH_DOES_NOT_EXIST 6
-
-/* Filehandle handling */
-#define IMAGICK_WRITE_IMAGE_FILE 1
-#define IMAGICK_WRITE_IMAGES_FILE 2
-#define IMAGICK_READ_IMAGE_FILE 3
-#define IMAGICK_PING_IMAGE_FILE 4
+typedef enum _php_imagick_rw_result_t {
+	IMAGICK_RW_OK,
+	IMAGICK_RW_SAFE_MODE_ERROR,
+	IMAGICK_RW_OPEN_BASEDIR_ERROR,
+	IMAGICK_RW_UNDERLYING_LIBRARY,
+	IMAGICK_RW_PERMISSION_DENIED,
+	IMAGICK_RW_FILENAME_TOO_LONG,
+	IMAGICK_RW_PATH_DOES_NOT_EXIST,
+	IMAGICK_RW_PATH_IS_DIR,
+} php_imagick_rw_result_t;
 
 /* Class entries */
 extern zend_class_entry *php_imagick_sc_entry;
@@ -164,6 +192,10 @@ extern zend_class_entry *php_imagickpixel_sc_entry;
 extern zend_class_entry *php_imagickpixel_exception_class_entry;
 extern zend_class_entry *php_imagickpixeliterator_sc_entry;
 extern zend_class_entry *php_imagickpixeliterator_exception_class_entry;
+#ifdef IMAGICK_WITH_KERNEL
+extern zend_class_entry *php_imagickkernel_sc_entry;
+extern zend_class_entry *php_imagickkernel_exception_class_entry;
+#endif
 
 /* Forward declarations (Imagick) */
 
@@ -179,7 +211,9 @@ PHP_METHOD(imagick, vignetteimage);
 PHP_METHOD(imagick, compareimagelayers);
 PHP_METHOD(imagick, optimizeimagelayers);
 PHP_METHOD(imagick, uniqueimagecolors);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagematte);
+#endif
 PHP_METHOD(imagick, setimagematte);
 PHP_METHOD(imagick, adaptiveresizeimage);
 PHP_METHOD(imagick, sketchimage);
@@ -190,7 +224,7 @@ PHP_METHOD(imagick, adaptiveblurimage);
 PHP_METHOD(imagick, contraststretchimage);
 PHP_METHOD(imagick, adaptivesharpenimage);
 PHP_METHOD(imagick, randomthresholdimage);
-PHP_METHOD(imagick, roundcorners);
+PHP_METHOD(imagick, roundcornersimage);
 PHP_METHOD(imagick, setiteratorindex);
 PHP_METHOD(imagick, getiteratorindex);
 PHP_METHOD(imagick, transformimage);
@@ -203,6 +237,8 @@ PHP_METHOD(imagick, orderedposterizeimage);
 PHP_METHOD(imagick, polaroidimage);
 PHP_METHOD(imagick, getimageproperty);
 PHP_METHOD(imagick, setimageproperty);
+PHP_METHOD(imagick, deleteimageproperty);
+PHP_METHOD(imagick, identifyformat);
 PHP_METHOD(imagick, setimageinterpolatemethod);
 PHP_METHOD(imagick, getimageinterpolatemethod);
 PHP_METHOD(imagick, linearstretchimage);
@@ -213,9 +249,11 @@ PHP_METHOD(imagick, extentimage);
 PHP_METHOD(imagick, getimageorientation);
 PHP_METHOD(imagick, setimageorientation);
 #endif
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 #if MagickLibVersion > 0x634
 PHP_METHOD(imagick, paintfloodfillimage);
 #endif
+#endif
 #if MagickLibVersion > 0x635
 PHP_METHOD(imagick, clutimage);
 PHP_METHOD(imagick, getimageproperties);
@@ -227,8 +265,10 @@ PHP_METHOD(imagick, resetimagepage);
 PHP_METHOD(imagick, setimageclipmask);
 PHP_METHOD(imagick, getimageclipmask);
 PHP_METHOD(imagick, animateimages);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, recolorimage);
 #endif
+#endif
 #if MagickLibVersion > 0x636
 PHP_METHOD(imagick, setfont);
 PHP_METHOD(imagick, getfont);
@@ -274,8 +314,31 @@ PHP_METHOD(imagick, exportimagepixels);
 PHP_METHOD(imagick, getimagechannelkurtosis);
 PHP_METHOD(imagick, functionimage);
 #endif
+#if MagickLibVersion > 0x651
+PHP_METHOD(imagick, transformimagecolorspace);
+#endif
+#if MagickLibVersion > 0x652
+PHP_METHOD(imagick, haldclutimage);
+#endif
+#if MagickLibVersion > 0x655
+PHP_METHOD(imagick, autolevelimage);
+PHP_METHOD(imagick, blueshiftimage);
+#endif
+#if MagickLibVersion > 0x656
+PHP_METHOD(imagick, setimageartifact);
+PHP_METHOD(imagick, getimageartifact);
+PHP_METHOD(imagick, deleteimageartifact);
+PHP_METHOD(imagick, setcolorspace);
+PHP_METHOD(imagick, getcolorspace);
+PHP_METHOD(imagick, clampimage);
+#endif
+#if MagickLibVersion > 0x667
+PHP_METHOD(imagick, smushimages);
+#endif
+
 PHP_METHOD(imagick, __construct);
 PHP_METHOD(imagick, __tostring);
+PHP_METHOD(imagick, count);
 PHP_METHOD(imagick, getpixeliterator);
 PHP_METHOD(imagick, getpixelregioniterator);
 PHP_METHOD(imagick, readimage);
@@ -298,14 +361,18 @@ PHP_METHOD(imagick, previousimage);
 PHP_METHOD(imagick, nextimage);
 PHP_METHOD(imagick, haspreviousimage);
 PHP_METHOD(imagick, hasnextimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, setimageindex);
 PHP_METHOD(imagick, getimageindex);
+#endif
 PHP_METHOD(imagick, setimageformat);
 PHP_METHOD(imagick, setimagefilename);
 PHP_METHOD(imagick, getimagefilename);
 PHP_METHOD(imagick, getimageformat);
 PHP_METHOD(imagick, getimagemimetype);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagesize);
+#endif
 PHP_METHOD(imagick, getimagegeometry);
 PHP_METHOD(imagick, getimagecolors);
 PHP_METHOD(imagick, scaleimage);
@@ -322,10 +389,13 @@ PHP_METHOD(imagick, getimageblob);
 PHP_METHOD(imagick, getimagesblob);
 PHP_METHOD(imagick, setimagecompressionquality);
 PHP_METHOD(imagick, getimagecompressionquality);
+PHP_METHOD(imagick, setimagecompression);
+PHP_METHOD(imagick, getimagecompression);
 PHP_METHOD(imagick, annotateimage);
 PHP_METHOD(imagick, compositeimage);
 PHP_METHOD(imagick, modulateimage);
 PHP_METHOD(imagick, montageimage);
+PHP_METHOD(imagick, morphology);
 PHP_METHOD(imagick, identifyimage);
 PHP_METHOD(imagick, thresholdimage);
 PHP_METHOD(imagick, adaptivethresholdimage);
@@ -345,7 +415,9 @@ PHP_METHOD(imagick, rotateimage);
 PHP_METHOD(imagick, sampleimage);
 PHP_METHOD(imagick, solarizeimage);
 PHP_METHOD(imagick, shadowimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, setimageattribute);
+#endif
 PHP_METHOD(imagick, setimagebackgroundcolor);
 PHP_METHOD(imagick, setimagecompose);
 PHP_METHOD(imagick, setimagecompression);
@@ -356,6 +428,7 @@ PHP_METHOD(imagick, setimageiterations);
 PHP_METHOD(imagick, setimagemattecolor);
 PHP_METHOD(imagick, setimagepage);
 PHP_METHOD(imagick, setimageprogressmonitor);
+PHP_METHOD(imagick, setprogressmonitor);
 PHP_METHOD(imagick, setimageresolution);
 PHP_METHOD(imagick, setimagescene);
 PHP_METHOD(imagick, setimagetickspersecond);
@@ -373,18 +446,24 @@ PHP_METHOD(imagick, queryfonts);
 PHP_METHOD(imagick, queryfontmetrics);
 PHP_METHOD(imagick, steganoimage);
 PHP_METHOD(imagick, motionblurimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, mosaicimages);
+#endif
 PHP_METHOD(imagick, morphimages);
 PHP_METHOD(imagick, minifyimage);
 PHP_METHOD(imagick, addnoiseimage);
 PHP_METHOD(imagick, affinetransformimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, averageimages);
+#endif
 PHP_METHOD(imagick, borderimage);
 PHP_METHOD(imagick, chopimage);
 PHP_METHOD(imagick, clipimage);
 PHP_METHOD(imagick, clippathimage);
 PHP_METHOD(imagick, coalesceimages);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, colorfloodfillimage);
+#endif
 PHP_METHOD(imagick, colorizeimage);
 PHP_METHOD(imagick, compareimagechannels);
 PHP_METHOD(imagick, compareimages);
@@ -399,20 +478,29 @@ PHP_METHOD(imagick, embossimage);
 PHP_METHOD(imagick, enhanceimage);
 PHP_METHOD(imagick, equalizeimage);
 PHP_METHOD(imagick, evaluateimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, flattenimages);
+#endif
 PHP_METHOD(imagick, flipimage);
 PHP_METHOD(imagick, flopimage);
+#if MagickLibVersion >= 0x655
+PHP_METHOD(imagick, forwardfouriertransformimage);
+#endif
 PHP_METHOD(imagick, frameimage);
 PHP_METHOD(imagick, fximage);
 PHP_METHOD(imagick, gammaimage);
 PHP_METHOD(imagick, gaussianblurimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimageattribute);
+#endif
 PHP_METHOD(imagick, getimagebackgroundcolor);
 PHP_METHOD(imagick, getimageblueprimary);
 PHP_METHOD(imagick, getimagebordercolor);
 PHP_METHOD(imagick, getimagechanneldepth);
 PHP_METHOD(imagick, getimagechanneldistortion);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagechannelextrema);
+#endif
 PHP_METHOD(imagick, getimagechannelmean);
 PHP_METHOD(imagick, getimagechannelstatistics);
 PHP_METHOD(imagick, getimagecolormapcolor);
@@ -421,7 +509,9 @@ PHP_METHOD(imagick, getimagecompose);
 PHP_METHOD(imagick, getimagedelay);
 PHP_METHOD(imagick, getimagedepth);
 PHP_METHOD(imagick, getimagedistortion);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimageextrema);
+#endif
 PHP_METHOD(imagick, getimagedispose);
 PHP_METHOD(imagick, getimagegamma);
 PHP_METHOD(imagick, getimagegreenprimary);
@@ -448,14 +538,21 @@ PHP_METHOD(imagick, getnumberimages);
 PHP_METHOD(imagick, getimagetotalinkdensity);
 PHP_METHOD(imagick, getimageregion);
 PHP_METHOD(imagick, implodeimage);
+#if MagickLibVersion >= 0x658
+PHP_METHOD(imagick, inversefouriertransformimage);
+#endif
 PHP_METHOD(imagick, levelimage);
 PHP_METHOD(imagick, magnifyimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, mapimage);
 PHP_METHOD(imagick, mattefloodfillimage);
+#endif
 PHP_METHOD(imagick, medianfilterimage);
 PHP_METHOD(imagick, negateimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, paintopaqueimage);
 PHP_METHOD(imagick, painttransparentimage);
+#endif
 PHP_METHOD(imagick, previewimages);
 PHP_METHOD(imagick, profileimage);
 PHP_METHOD(imagick, quantizeimage);
@@ -465,6 +562,7 @@ PHP_METHOD(imagick, removeimageprofile);
 PHP_METHOD(imagick, separateimagechannel);
 PHP_METHOD(imagick, sepiatoneimage);
 PHP_METHOD(imagick, setimagebias);
+PHP_METHOD(imagick, setimagebiasquantum);
 PHP_METHOD(imagick, setimageblueprimary);
 PHP_METHOD(imagick, setimagebordercolor);
 PHP_METHOD(imagick, setimagechanneldepth);
@@ -499,6 +597,7 @@ PHP_METHOD(imagick, getinterlacescheme);
 PHP_METHOD(imagick, getoption);
 PHP_METHOD(imagick, getpackagename);
 PHP_METHOD(imagick, getpage);
+PHP_METHOD(imagick, getquantum);
 PHP_METHOD(imagick, getquantumdepth);
 PHP_METHOD(imagick, getquantumrange);
 PHP_METHOD(imagick, getreleasedate);
@@ -520,6 +619,29 @@ PHP_METHOD(imagick, setresolution);
 PHP_METHOD(imagick, setsamplingfactors);
 PHP_METHOD(imagick, setsize);
 PHP_METHOD(imagick, settype);
+PHP_METHOD(imagick, brightnesscontrastimage);
+#if MagickLibVersion > 0x661
+PHP_METHOD(imagick, colormatriximage);
+#endif
+PHP_METHOD(imagick, selectiveblurimage);
+#if MagickLibVersion >= 0x689
+PHP_METHOD(imagick, rotationalblurimage);
+#endif
+#if MagickLibVersion >= 0x683
+PHP_METHOD(imagick, statisticimage);
+#endif
+#if MagickLibVersion >= 0x652
+PHP_METHOD(imagick, subimagematch);
+#endif
+
+PHP_METHOD(imagick, setregistry);
+PHP_METHOD(imagick, getregistry);
+PHP_METHOD(imagick, listregistry);
+PHP_METHOD(imagick, morphology);
+PHP_METHOD(imagick, filter);
+PHP_METHOD(imagick, setantialias);
+PHP_METHOD(imagick, getantialias);
+
 
 /* Forward declarations (ImagickDraw) */
 #if MagickLibVersion > 0x628
@@ -531,9 +653,14 @@ PHP_METHOD(imagickdraw, settextkerning);
 PHP_METHOD(imagickdraw, gettextinterwordspacing);
 PHP_METHOD(imagickdraw, settextinterwordspacing);
 #endif
+#if MagickLibVersion > 0x655
+PHP_METHOD(imagickdraw, gettextinterlinespacing);
+PHP_METHOD(imagickdraw, settextinterlinespacing);
+#endif
 PHP_METHOD(imagickdraw, __construct);
 PHP_METHOD(imagickdraw, setfillcolor);
 PHP_METHOD(imagickdraw, setfillalpha);
+PHP_METHOD(imagickdraw, setresolution);
 PHP_METHOD(imagickdraw, setstrokecolor);
 PHP_METHOD(imagickdraw, setstrokealpha);
 PHP_METHOD(imagickdraw, setstrokewidth);
@@ -652,6 +779,8 @@ PHP_METHOD(imagickdraw, push);
 PHP_METHOD(imagickpixeliterator, __construct);
 PHP_METHOD(imagickpixeliterator, newpixeliterator);
 PHP_METHOD(imagickpixeliterator, newpixelregioniterator);
+PHP_METHOD(imagickpixeliterator, getpixeliterator);
+PHP_METHOD(imagickpixeliterator, getpixelregioniterator);
 PHP_METHOD(imagickpixeliterator, getiteratorrow);
 PHP_METHOD(imagickpixeliterator, setiteratorrow);
 PHP_METHOD(imagickpixeliterator, getpreviousiteratorrow);
@@ -678,13 +807,26 @@ PHP_METHOD(imagickpixel, __construct);
 PHP_METHOD(imagickpixel, setcolor);
 PHP_METHOD(imagickpixel, clear);
 PHP_METHOD(imagickpixel, destroy);
-PHP_METHOD(imagickpixel, issimilar);
+PHP_METHOD(imagickpixel, ispixelsimilarquantum);
+PHP_METHOD(imagickpixel, ispixelsimilar);
 PHP_METHOD(imagickpixel, getcolorvalue);
 PHP_METHOD(imagickpixel, setcolorvalue);
 PHP_METHOD(imagickpixel, getcolor);
+PHP_METHOD(imagickpixel, getcolorquantum);
 PHP_METHOD(imagickpixel, getcolorasstring);
 PHP_METHOD(imagickpixel, getcolorcount);
 PHP_METHOD(imagickpixel, setcolorcount);
 PHP_METHOD(imagickpixel, clone);
 
+#ifdef IMAGICK_WITH_KERNEL
+PHP_METHOD(imagickkernel, frommatrix);
+PHP_METHOD(imagickkernel, frombuiltin);
+PHP_METHOD(imagickkernel, addkernel);
+PHP_METHOD(imagickkernel, getmatrix);
+PHP_METHOD(imagickkernel, separate);
+PHP_METHOD(imagickkernel, scale);
+PHP_METHOD(imagickkernel, addunitykernel);
+#endif
+
+
 #endif /* PHP_IMAGICK_DEFS_H */
